create view Employees as 
select
  [Employee Name],
  Division,
  [2003 Sales],
  Manager,
  Rating
-- (provider, connection string, table)
-- HDR=YES says first row is column names
from OpenRowset(
  'Microsoft.Jet.OLEDB.4.0',
  'Excel 8.0;Database=c:\excel\DoMore.xls;HDR=YES',
  Employees$)
go



create view DivisionSalesGuide as
select
  Division,
  cast(avg([2003 Sales]) as decimal(12,2)) avgSales,
  cast(stdevp([2003 Sales]) as decimal(12,2)) stdevSales
from OpenRowset(
  'Microsoft.Jet.OLEDB.4.0',
  'Excel 8.0;Database=c:\excel\DoMore.xls;HDR=YES',
  Employees$)
group by Division
go
-- helps us measure how extraordinary (good or bad) employees are.
select * from DivisionSalesGuide

create view ManagerPerformanceGuide as
select
  Manager,
  cast(avg([Rating]) as decimal(12,2)) avgPerformance,
  cast(stdevp([Rating]) as decimal(12,2)) stdevPerformance
from OpenRowset(
  'Microsoft.Jet.OLEDB.4.0',
  'Excel 8.0;Database=c:\excel\DoMore.xls;HDR=YES',
  Employees$)
group by Manager
go

--select * from DivisionSalesGuide
--select * from ManagerPerformanceGuide
go

create view EmployeeScores as
  select
    E.[Employee Name],
    E.Division,
    E.Manager,
    cast((E.[2003 Sales] - DSG.avgSales)/
       isnull(nullif(DSG.stdevSales,0),1.00) as decimal(6,3)) SalesScore,
-- standardized ratings of sales
    cast((E.Rating - MPG.avgPerformance)/
       isnull(nullif(MPG.stdevPerformance,0),1.00) as decimal(6,3)) PerformanceScore
-- and performance (manager-rated)
  from DivisionSalesGuide DSG, ManagerPerformanceGuide MPG, Employees E
  where E.Manager = MPG.Manager
  and E.Division = DSG.Division
go
-- select * from EmployeeScores


create view TopQuarterEmployees as
  select top 25 percent with ties
    [Employee Name],
    0.3*SalesScore + 0.7*PerformanceScore as EmployeeScore
-- weighted performance evaluation: 70% manager, 30% sales
  from EmployeeScores
  order by 2 desc
go

select * from TopQuarterEmployees
go
/*
select
  Division,
  count(topQuarter),
  count(*) as totalCount,
  1.0*count(topQuarter)/count(*) as perCentage
from (
  select
    Manager,
    Division,
    case when [Employee Name] in (select [Employee Name] from TopQuarterEmployees) then 1 end topQuarter
  from EmployeeScores
) T
group by Division
*/
go

drop view
  TopQuarterEmployees,
  EmployeeScores,
  DivisionSalesGuide,
  ManagerPerformanceGuide,
  Employees