set nocount on
go
set statistics io off
set statistics time off
go

use Connections
go

create table Items (
  pkItem    int identity(1,1) primary key,
  born      smallint,
  gender    char,
  state     char(2),
  heightcm  smallint,
  income    int,
  children  tinyint,
  dogs      tinyint,
  cats      tinyint,
  autos     tinyint,
  homes     tinyint,
  yrsSchool tinyint
)
go
declare @i int
set @i = 1000 -- 00
while @i > 0 begin
  insert into Items values (
    1880+rand()*123,
    substring('FFFFFFFFFFFFFFFMMMMMMMMMMMMMM?',convert(int,1+30*RAND()),1),
    substring('ALAKASAZARCACOCTDEDCFMFLGAGUHIIDILINIAKSKYLAMEMHMDMAMIMNMSMOMTNENVNHNJNMNYNCNDMPOHOKORPWPAPRRISCSDTNTXUTVTVIVAWAWVWIWY??',
      convert(int,1+2*floor(rand()*60)),2),
    (1-rand()*rand())*180 + rand()*70,
    (1-rand())*65000 + rand()*rand()*200000,
    6-5*rand()*rand(),
    rand()*rand()*rand()*7,
    rand()*rand()*rand()*7,
    rand()*rand()*rand()*4,
    rand()*rand()*3,
    6 + rand()*18
  )
  set @i = @i - 1
end
go

-- select * from Items

create table Properties (
  pkProperty  smallint identity,
  PropDesc    varchar(100) primary key,
  LowVal      sql_variant,
  HighVal     sql_variant
)
go
create unique nonclustered index Properties_propKey on Properties(pkProperty)

exec master..xp_regwrite
     @rootkey='HKEY_LOCAL_MACHINE',
     @key='SOFTWARE\Microsoft\Jet\4.0\Engines\Excel',
     @value_name='TypeGuessRows',
     @type='REG_DWORD',
     @value=200
go
exec master..xp_regwrite
     @rootkey='HKEY_LOCAL_MACHINE',
     @key='SOFTWARE\Microsoft\Jet\4.0\Engines\Excel',
     @value_name='ImportMixedTypes',
     @type='REG_SZ',
     @value='Text'
go

insert into Properties(PropDesc,LowVal,HighVal)
select
  Property,
  case when LowVal is null
         then cast(1 as sql_variant)
       when LowVal not like '%[^0-9]%'
         then cast(cast(LowVal as int) as sql_variant)
       else cast(cast(LowVal as varchar(100)) as sql_variant)
  end,
  case when HighVal is null
         then cast(1 as sql_variant)
       when HighVal not like '%[^0-9]%'
         then cast(cast(HighVal as int) as sql_variant)
       else cast(cast(HighVal as varchar(100)) as sql_variant)
  end
from OpenRowSet(
  'Microsoft.Jet.OLEDB.4.0',
  'Excel 8.0;Database=c:\excel\Properties.xls;HDR=YES;IMEX=1'
  ,'select * from Properties'
)
go

-- select * from Properties

-- key to efficiency:  precalculated satisfaction criteria.
create table PropertyIndex (
  pkProperty int not null,
  pkItem     int not null
  primary key (pkProperty, pkItem),
  unique (pkItem, pkProperty)
)
go

insert into PropertyIndex
select
  pkProperty,
  pkItem
from Items, Properties
where
  case when PropDesc = 'Never' then cast(1 as sql_variant)
    when PropDesc like 'State%' then state
    when PropDesc in ('Male','Female') then gender
    when PropDesc like 'Born%' then born
    when PropDesc like 'Height%' then heightcm
    when PropDesc like 'Income%' then income
    when PropDesc like 'Children%' then children
    when PropDesc like 'Dogs%' then dogs
    when PropDesc like 'Cats%' then cats
    when PropDesc like 'Autos%' then autos
    when PropDesc like 'Homes%' then homes
    when PropDesc like 'Yrs%' then yrsSchool
  end between LowVal and HighVal
go

set statistics io on
set statistics time on
go


select
  I1.pkItem
from
  PropertyIndex I1,
  PropertyIndex I2,
  PropertyIndex I3,
  PropertyIndex I4
where
    I1.pkProperty = 97
and I2.pkProperty = 103
and (I3.pkProperty = 112 or I3.pkProperty = 113)
and I4.pkProperty = 120
and I1.pkItem = I2.pkItem
and I2.pkItem = I3.pkItem
and I3.pkItem = I4.pkItem
go

-- I want a table-valued function that lets me
-- pass a string of criteria to
-- match and have all the matching items' keys returned as
-- a table:  Example:
/*
select pkItem from
f_Props
('IncomeTo50000       Male                ')
*/
create function f_Props (
  @matchList varchar(8000)
) returns @t table (
    pkItem int primary key
) as begin
  declare @l int
  set @l = datalength(@matchList)
  declare @left varchar(4000), @right varchar(4000)
  if datalength(@matchList) <= 20 begin
    insert into @t
      select I.pkItem
      from PropertyIndex I, Properties P
      where P.pkProperty = I.pkProperty
      and PropDesc = right(@matchList,20)
    return
-- above is the easy result for a one-criterion string.

  end
  set @left  = substring(@matchList,1,(@l+@l%40)/2)
  set @right = substring(@matchList,(@l+@l%40)/2+1,(@l-@l%40)/2)
-- find stuff satisfying half the criteria, then the other half,
-- then return results appear in both searches.
  insert into @t
    select I.pkItem
    from PropertyIndex I
    join Properties P
    on I.pkProperty = P.pkProperty
    and P.PropDesc = right(@matchList,20)
    join f_Props(@left) L
    on I.pkItem = L.pkItem
    join f_Props(@right) R
    on I.pkItem = R.pkItem
  return
end
go
select * from Items
declare @d datetime, @s datetime
set @d = getdate()
select * from
f_Props
('IncomeTo50000       Male                IncomeTo20000       ')
set @s = getdate()
select datediff(ms,@d,@s)

go



declare @d datetime, @s datetime
set @d = getdate()
select
  I1.pkItem
from
  PropertyIndex I1 inner join PropertyIndex I2
on I1.pkItem = I2.pkItem
where
    I1.pkProperty = 97
and I2.pkProperty = 103

set @s = getdate()
select datediff(ms,@d,@s)
go

select
  I1.pkItem
from
  PropertyIndex I1 inner join PropertyIndex I2
on I1.pkItem = I2.pkItem inner join PropertyIndex I3
on I2.pkItem = I3.pkItem inner join PropertyIndex I4
on I3.pkItem = I4.pkItem
where
    I1.pkProperty = 97
and I2.pkProperty = 103
and I3.pkProperty = 112
and I4.pkProperty = 120
go


select
  I1.pkItem
from
  PropertyIndex I1 inner merge join PropertyIndex I2
on I1.pkItem = I2.pkItem inner merge join PropertyIndex I3
on I2.pkItem = I3.pkItem inner merge join PropertyIndex I4
on I3.pkItem = I4.pkItem
where
    I1.pkProperty = 112
and I2.pkProperty = 103
and I3.pkProperty = 97
and I4.pkProperty = 120
go


select pkItem from Items
where
  income between 20001 and 50000
  and children = 2
  and (cats = 1)
  and homes = 1

go

set statistics io off
set statistics time off
go

drop table Properties
drop table PropertyIndex
drop table Items
go

drop function f_Props
go